#include "create.h"
#include "aswm/log/log.h"
#include <X11/Xutil.h>

void OnCreateNotify(Aswm* aswm, const XCreateWindowEvent* e) {
	aswm_info("XCreateWindowEvent %lu on parent %lu", e->window, e->parent);
	aswm_info("\tx = %i", e->x);
	aswm_info("\ty = %i", e->y);
	aswm_info("\tw = %i", e->width);
	aswm_info("\th = %i", e->height);
	aswm_info("\toverride_redirect = %i", e->override_redirect);

	XWMHints* hints = XGetWMHints(aswm->display, e->window);
	if(hints != NULL) {
		aswm_info("\tWM_HINTS");
		aswm_info("\t\tinitial_state: %i", hints->initial_state);
	}

	// The WM must subscribe to property changes, notably to implement the
	// Extended Window Manager Hints (EWMH) propocol
	XSelectInput(aswm->display, e->window, PropertyChangeMask);

	{
		XTextProperty text_property;
		if(XGetWMName(e->display, e->window, &text_property)) {
			aswm_info("\tname: %s", text_property.value);
		}
	}
	{
		XClassHint class;
		if(XGetClassHint(e->display, e->window, &class)) {
			aswm_info("\tclass: %s, %s", class.res_class, class.res_name);
			XFree(class.res_class);
			XFree(class.res_name);
		}
	}
	if(aswm->desktops_count == 0)
		aswm_create_workspace();
}
