#include "property.h"
#include <string.h>
#include "aswm/mapper/mapper.h"
#include "aswm/log/log.h"
#include <X11/Xutil.h>
#include <X11/Xatom.h>

void OnPropertyNotify(Aswm* aswm, XPropertyEvent* e) {
	aswm_notice("Property change for window %lu:", e->window);
	char* property_name = XGetAtomName(e->display, e->atom);
	aswm_notice("\t%s", property_name);
	if(strcmp(property_name, "_NET_WM_NAME") == 0) {
		aswm_notice("\t-> Unhandled");
		XTextProperty wm_name_property;
		char** wm_name;
		int wm_name_count;
		XGetTextProperty(e->display, e->window, &wm_name_property, e->atom);
		Xutf8TextPropertyToTextList(e->display, &wm_name_property, &wm_name, &wm_name_count);
		for(auto i = 0; i < wm_name_count; i++)
			aswm_notice("\t_NET_WM_NAME: %s", wm_name[i]);
		XFreeStringList(wm_name);
	} else if(strcmp(property_name, "WM_NAME") == 0) {
		aswm_notice("\t-> Unhandled");
		XTextProperty text_property;
		if(XGetWMName(e->display, e->window, &text_property)) {
			aswm_notice("\tWM_NAME: %s", text_property.value);
		}
	} else if(strcmp(property_name, "WM_NORMAL_HINTS") == 0) {
		XSizeHints* size = XAllocSizeHints();
			long supplied;
			if(XGetWMNormalHints(e->display, e->window, size, &supplied)) {
				aswm_notice("\tWMNormalHints:");
				aswm_notice("\t\tMin w:%i", size->min_width);
				aswm_notice("\t\tMin h:%i", size->min_height);
				aswm_notice("\t\tMax w:%i", size->max_width);
				aswm_notice("\t\tMax h:%i", size->max_height);
				aswm_notice("\t\tBase w:%i", size->base_width);
				aswm_notice("\t\tBase h:%i", size->base_height);
				aswm_handle_normal_hints(
						aswm->display, aswm->current_desktop.stack_root,
						e->window, size, supplied);
			}
			XFree(size);
	} else if(strcmp(property_name, "WM_HINTS") == 0) {
		aswm_notice("\t-> Unhandled");
		XWMHints* hints = XGetWMHints(aswm->display, e->window);
		aswm_notice("\t\tflags Input: %li", hints->flags & InputHint);
		aswm_notice("\t\tflags Input: %li", hints->flags & StateHint);
		aswm_notice("\t\tflags IconPixmap: %li", hints->flags & IconPixmapHint);
		aswm_notice("\t\tflags WindowHint: %li", hints->flags & IconWindowHint);
		aswm_notice("\t\tflags IconPosition: %li", hints->flags & IconPositionHint);
		aswm_notice("\t\tflags IconMask: %li", hints->flags & IconMaskHint);
		aswm_notice("\t\tflags WindowGroup: %li", hints->flags & WindowGroupHint);
		aswm_notice("\t\tflags XUrgency: %li", hints->flags & XUrgencyHint);
		aswm_notice("\t\tflags AllHints: %li", hints->flags & AllHints);
		aswm_notice("\t\tinitial_state: %i", hints->initial_state);
		XFree(hints);
	} else if(strcmp(property_name, "WM_PROTOCOLS") == 0) {
		aswm_notice("\t-> Unhandled");
		Atom* protocols;
		int count;
		XGetWMProtocols(aswm->display, e->window, &protocols, &count);
		if(count > 0)
			aswm_notice("\t\t");
		for(auto i = 0; i < count; i++) {
			char* name = XGetAtomName(aswm->display, protocols[i]);
			aswm_notice("%s ", name);
			XFree(name);
		}
		if(count > 0)
			aswm_notice("");
		XFree(protocols);

	} else {
		aswm_notice("\t-> Unhandled");
	}
	XFree(property_name);
}
