#include "unmap.h"
#include <stdio.h>
#include "aswm/log/log.h"
#include <X11/Xutil.h>

void OnUnmapNotify(Aswm* aswm, const XUnmapEvent* e) {
	aswm_notice("XUnmapWindow %lu tiled in %lu", e->window, e->event);
	XWMHints* hints = XGetWMHints(aswm->display, e->window);
	if(hints != NULL) {
		aswm_notice("\tWM_HINTS");
		aswm_notice("\t\tinitial_state: %i", hints->initial_state);
	}
	// Since SubstructureNotify is selected on each frame, e->event corresponds
	// to the parent of the unmapped window (e->window)

	// Unmap should have no effect on not map windows. Currently, all mapped
	// windows are children of aswm->root_window.
	if(e->event != XDefaultRootWindow(e->display)) {
		if(e->event == aswm->current_desktop.tile_root) {
			aswm_notice("aswm untile");
			aswm_untile_window(e->display, e->event, e->window);
		} else if(e->event == aswm->current_desktop.stack_root) {
			aswm_notice("aswm unstack");
			aswm_unstack_window(e->display, e->event);
		}
		aswm_info("Unmap tree:");
		aswm_log_tree(aswm_info, aswm, DefaultRootWindow(aswm->display), 1);
	}
}
