#include "log.h"
#include <stdio.h>

zlog_category_t* aswm_cat;

_ASWM_LOG_TREE_DEF(aswm_info)
_ASWM_LOG_TREE_DEF(aswm_warn)
_ASWM_LOG_TREE_DEF(aswm_debug)
_ASWM_LOG_TREE_DEF(aswm_notice)
_ASWM_LOG_TREE_DEF(aswm_fatal)

void log_tree(Aswm* aswm, Window window, int indent) {
	XWindowAttributes window_attributes;
	XGetWindowAttributes(aswm->display, window, &window_attributes);
	for(int i = 0; i < indent; i++)
		printf("  ");
	printf("%lu ", window);
	if(window == XDefaultRootWindow(aswm->display)) {
		printf("(X root) ");
	} else if (window == aswm->root_window) {
		printf("(ASWM root) ");
	} else if (window == aswm->current_desktop.root_window) {
		printf("(current workspace) ");
	} else if (window == aswm->current_desktop.tile_root) {
		printf("(tile root) ");
	} else if (window == aswm->current_desktop.stack_root) {
		printf("(stack root) ");
	}
	printf("at (%i, %i): %ix%i\n", window_attributes.x, window_attributes.y, window_attributes.width, window_attributes.height);
	
	Window root_window;
	Window parent_tile;
	Window* children;
	unsigned int children_count;
	XQueryTree(aswm->display, window, &root_window, &parent_tile, &children, &children_count);
	for(auto i = 0; i < children_count; i++)
		log_tree(aswm, children[i], indent+1);
	XFree(children);
}

void set_up_logger(const char* zlog_conf) {
	int rc;

	rc = zlog_init(zlog_conf);
	if (rc) {
		printf("Init zlog from %s failed\n", zlog_conf);
	}

	aswm_cat = zlog_get_category("aswm");
	if (!aswm_cat) {
		printf("Get zlog aswm category failed\n");
	}
}

void tear_down_logger(void) {
	zlog_fini();
}

