#ifndef ASWM_LOG
#define ASWM_LOG

#include <X11/Xlib.h>
#include "aswm/aswm.h"
#include "zlog.h"
#include <string.h>

extern zlog_category_t* aswm_cat;

#define aswm_fatal(format, ...) zlog_fatal(aswm_cat, format __VA_OPT__(,) __VA_ARGS__)
#define aswm_error(format, ...) zlog_error(aswm_cat, format __VA_OPT__(,) __VA_ARGS__)
#define aswm_warn(format, ...) zlog_warn(aswm_cat, format __VA_OPT__(,) __VA_ARGS__)
#define aswm_notice(format, ...) zlog_notice(aswm_cat, format __VA_OPT__(,) __VA_ARGS__)
#define aswm_info(format, ...) zlog_info(aswm_cat, format __VA_OPT__(,) __VA_ARGS__)
#define aswm_debug(format, ...) zlog_debug(aswm_cat, format __VA_OPT__(,) __VA_ARGS__)

void set_up_logger(const char* zlog_conf);
void tear_down_logger();

void log_tree(Aswm* aswm, Window window, int indent);
	
#define _ASWM_LOG_TREE_DEC(logger)\
	void _aswm_log_tree_##logger(Aswm* aswm, Window window, int indent);

#define _ASWM_LOG_TREE_DEF(logger)\
	void _aswm_log_tree_##logger(Aswm* aswm, Window window, int indent) {\
		XWindowAttributes window_attributes;\
		XGetWindowAttributes(aswm->display, window, &window_attributes);\
		char format[80];\
		strcpy(format, "");\
		for(int i = 0; i < indent; i++)\
			strcat(format, "  ");\
		strcat(format, "%lu ");\
		if(window == XDefaultRootWindow(aswm->display)) {\
			strcat(format,"(X root) ");\
		} else if (window == aswm->root_window) {\
			strcat(format,"(ASWM root) ");\
		} else if (window == aswm->current_desktop.root_window) {\
			strcat(format,"(current workspace) ");\
		} else if (window == aswm->current_desktop.tile_root) {\
			strcat(format,"(tile root) ");\
		} else if (window == aswm->current_desktop.stack_root) {\
			strcat(format,"(stack root) ");\
		}\
		strcat(format, "at (%i, %i): %ix%i");\
		logger(format, window, window_attributes.x, window_attributes.y, window_attributes.width, window_attributes.height);\
		Window root_window;\
		Window parent_tile;\
		Window* children;\
		unsigned int children_count;\
		XQueryTree(aswm->display, window, &root_window, &parent_tile, &children, &children_count);\
		for(auto i = 0; i < children_count; i++)\
			_aswm_log_tree_##logger(aswm, children[i], indent+1);\
		XFree(children);\
	}

_ASWM_LOG_TREE_DEC(aswm_info)
_ASWM_LOG_TREE_DEC(aswm_warn)
_ASWM_LOG_TREE_DEC(aswm_debug)
_ASWM_LOG_TREE_DEC(aswm_notice)
_ASWM_LOG_TREE_DEC(aswm_fatal)

#define aswm_log_tree(logger, aswm, window, indent) _aswm_log_tree_##logger(aswm, window, indent)

#endif
