#include "test_aswm_create_windows.h"

#include <time.h>

#include "aswm/aswm.h"
#include "../test_set_up.h"

clock_t USER_DELAY = 1 * CLOCKS_PER_SEC;
#define MAX_COLOR 65535

void create_single_resizeable_top_level_window(void) {
	XColor blue;
	blue.blue = MAX_COLOR;
	XAllocColor(test_display, XDefaultColormap(test_display, DefaultScreen(test_display)), &blue);

	XColor red;
	red.red = MAX_COLOR;
	XAllocColor(test_display, XDefaultColormap(test_display, DefaultScreen(test_display)), &red);

	Window window = XCreateSimpleWindow(test_display, test_root,
			10 /*x*/, 10 /*y*/, 128 /*w*/, 128 /*h*/,
			0 /*TODO: border width*/, CopyFromParent /*depth*/, blue.pixel /* background */);
	// The top level window might have children. Those must **not** be altered
	// by the Window Manager
	Window child = XCreateSimpleWindow(test_display, window,
			0, 0, 25, 30,
			0, CopyFromParent, red.pixel);
	XSelectInput(test_display, window, VisibilityChangeMask);
	XMapWindow(test_display, child);
	XMapWindow(test_display, window);

	int visibility;
	clock_t start = clock();
	while((clock() - start) < USER_DELAY) {
		XEvent e;
		XCheckWindowEvent(test_display, window, VisibilityChangeMask, &e);
		visibility = e.xvisibility.state;
	}

	// The top level window if fully visible
	XWindowAttributes attributes;
	XGetWindowAttributes(test_display, window, &attributes);
	TEST_ASSERT_EQUAL_INT(VisibilityUnobscured, visibility);
	TEST_ASSERT_EQUAL_INT(IsViewable, attributes.map_state);

	// Check the child structure is unaltered
	Window root;
	Window parent;
	Window* children;
	unsigned int children_count;
	XQueryTree(test_display, child, &root, &parent, &children, &children_count);

	TEST_ASSERT_EQUAL(parent, window);

	XWindowAttributes child_attributes;
	XGetWindowAttributes(test_display, child, &child_attributes);

	TEST_ASSERT_EQUAL_INT(0, child_attributes.x);
	TEST_ASSERT_EQUAL_INT(0, child_attributes.y);
	TEST_ASSERT_EQUAL_INT(25, child_attributes.width);
	TEST_ASSERT_EQUAL_INT(30, child_attributes.height);
	TEST_ASSERT_EQUAL_INT(0, child_attributes.border_width);
	TEST_ASSERT_EQUAL_INT(IsViewable, child_attributes.map_state);
}

void create_multiple_resizeable_top_level_window(void) {
	XColor red;
	red.red = MAX_COLOR;
	XAllocColor(test_display, XDefaultColormap(test_display, DefaultScreen(test_display)), &red);

	const int N = 10;
	Window windows[N];
	Window children[N];

	for(auto i = 0; i < N; i++) {
		XColor blue;
		blue.blue = (i+1) * MAX_COLOR / N;
		XAllocColor(test_display, XDefaultColormap(test_display, DefaultScreen(test_display)), &blue);

		windows[i] = XCreateSimpleWindow(test_display, test_root,
				10 /*x*/, 10 /*y*/, 128 /*w*/, 128 /*h*/,
				0 /*TODO: border width*/, CopyFromParent /*depth*/, blue.pixel /* background */);
		// The top level windows might have children. Those must **not** be
		// altered by the Window Manager
		children[i] = XCreateSimpleWindow(test_display, windows[i],
				0, 0, 25, 30,
				0, CopyFromParent, red.pixel);
		XSelectInput(test_display, windows[i], VisibilityChangeMask);
		XMapWindow(test_display, children[i]);
		XMapWindow(test_display, windows[i]);
	}

	
	clock_t user_time = clock();
	int visibility[N];
	while((clock() - user_time) < USER_DELAY * N) {
		for(auto i = 0; i < N; i++) {
			XEvent e;
			XCheckWindowEvent(test_display, windows[i], VisibilityChangeMask, &e);
			visibility[i] = e.xvisibility.state;
		}
	}

	for(auto i = 0; i < N; i++) {
		// All top level windows must be fully visible
		XWindowAttributes attributes;
		XGetWindowAttributes(test_display, windows[i], &attributes);
		TEST_ASSERT_EQUAL_INT(VisibilityUnobscured, visibility[i]);
		TEST_ASSERT_EQUAL_INT(IsViewable, attributes.map_state);

		// Check the children structure is unaltered
		Window _root;
		Window parent;
		Window* _children;
		unsigned int _children_count;
		XQueryTree(test_display, children[i], &_root, &parent, &_children, &_children_count);

		TEST_ASSERT_EQUAL(parent, windows[i]);

		XWindowAttributes child_attributes;
		XGetWindowAttributes(test_display, children[i], &child_attributes);

		TEST_ASSERT_EQUAL_INT(0, child_attributes.x);
		TEST_ASSERT_EQUAL_INT(0, child_attributes.y);
		TEST_ASSERT_EQUAL_INT(25, child_attributes.width);
		TEST_ASSERT_EQUAL_INT(30, child_attributes.height);
		TEST_ASSERT_EQUAL_INT(0, child_attributes.border_width);
		TEST_ASSERT_EQUAL_INT(IsViewable, child_attributes.map_state);
	}
}

